//
//  BookmarksHandle.m
//  Dynamics
//
//  Created by Pierre d'Herbemont on Thu Jul 26 2001.
//		steg : steg@mac.com
//
//  This 2 classes provide a way to add a 'bookmark' system like to Mandelbrot's Dynamics
//	Note that I move the Inspector in a new inspector which use panel, and bookmarks...
//	The bookmarks can be exported (.dynbm), and they are saved in a sort of preferences
//	file in ~/Library/DynamicsPrefs...
//  note that I changed nothing in your code, I only changed some link in IB for the
//	MandelView instance, for the new Inspector window.
//------------------------------------------------------------------------------------------
//		
//	This class is the data source of the listbox
#import "BookmarksHandle.h"
#import "BookmarkItem.h"
#import "MandelView.h"
#import <Cocoa/Cocoa.h>
@implementation BookmarksHandle
/*initialization stuff*/
- (void)dealloc {
    [theBookmarks release];
    [super dealloc];
}
- (void)awakeFromNib {
    //here we can get saved bookmarks
    [self setBookmarks:(NSMutableArray *)[NSUnarchiver unarchiveObjectWithFile:[[NSString stringWithString:@"~/Library/DynamicsBookmarks.dynbm"] stringByExpandingTildeInPath]]];
    if([self theBookmarks]==NULL)
    {
        [self setBookmarks:[NSMutableArray array]];
    }
    else
    {
        [self setBookmarksMenu];
    }
}


/*useful for the data source connection*/
- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
    return [theBookmarks count];//Number Of Row
}
- (id)tableView:(NSTableView *)aTableView
    objectValueForTableColumn:(NSTableColumn *)aTableColumn
    row:(int)rowIndex
{
    NSString 		*identifier = [aTableColumn identifier];
    BookmarkItem	*aBookmark = [theBookmarks objectAtIndex:rowIndex];
    return [aBookmark valueForKey: identifier]; 
}
- (void)tableView:(NSTableView *)aTableView
    setObjectValue:anObject
    forTableColumn:(NSTableColumn *)aTableColumn
    row:(int)rowIndex
{
    NSString *identifier = [aTableColumn identifier];
    BookmarkItem *aBookmark = [theBookmarks objectAtIndex: rowIndex];
    [aBookmark takeValue: anObject forKey: identifier];
    [self setBookmarksMenu];
    [self saveBookmarks];
    if([self updateMoreInfoWithMandelView:YES]){}
            
}


/*core bookmarks stuff*/
- (NSArray *)theBookmarks {
    return theBookmarks;
}
- (void)setBookmarks:(NSMutableArray *)nBoookmarks {
    [theBookmarks release];
    theBookmarks=[nBoookmarks retain];
}

- (void)saveBookmarks {
    BOOL state;
    state=[NSArchiver archiveRootObject:theBookmarks 
                    toFile:[[NSString stringWithString:@"~/Library/DynamicsBookmarks.dynbm"] 
                    stringByExpandingTildeInPath]];
    if(!state)
        NSRunAlertPanel(NSLocalizedString(@"Error",@"Error, alert title"),
                        NSLocalizedString(@"An error accured during the saving process of the default Bookmarks.","error when the bookmarks were saved to the ~/Library/DynamicsBookmarks.dynbm file"),
                        NSLocalizedString(@"OK",@"OK."),nil,nil);
}
- (void)openBookmarks {
    NSOpenPanel *op;
    int answer;
    op=[NSOpenPanel openPanel];
    [op setTitle:NSLocalizedString(@"Open",@"OpenPanelTitle")];
    answer = [op runModalForTypes: [NSArray arrayWithObject:@"dynbm"]];
    if(answer == NSOKButton) {
        [self setBookmarks:(NSMutableArray *)
                            [NSUnarchiver unarchiveObjectWithFile:[op filename]]];
    }
    [self setBookmarksMenu];
    [bookmarksView reloadData];
    [self saveBookmarks];
}
- (void)saveBookmarksAs {
    BOOL state;
    int answer;
    NSSavePanel *sp;
    sp=[NSSavePanel savePanel];
    [sp setTitle:NSLocalizedString(@"Save",@"SavePanel title")];
    [sp setRequiredFileType:@"dynbm"];
    answer = [sp runModal];
    if(answer == NSOKButton) {
        state=[NSArchiver archiveRootObject:theBookmarks 
                        toFile:[sp filename]];
        if(!state)
            answer=NSRunAlertPanel(NSLocalizedString(@"Error","Error, alert title"),
                NSLocalizedString(@"An error occured during the saving process.",
                    @"An error occured during the saving process of the bookmarks, after the save as command"),
                NSLocalizedString(@"OK",@"OK."),nil,nil);
    }
}


/*Connection with the resources*/
- (void)addItem:sender {
    BookmarkItem *aBookmark = [[BookmarkItem alloc] init];
    [aBookmark setX:(NSDecimalNumber *)[NSDecimalNumber numberWithDouble:[aMandelView Cwx]]];
    [aBookmark setY:(NSDecimalNumber *)[NSDecimalNumber numberWithDouble:[aMandelView Cwy]]];
    [aBookmark setMagnitude:(NSDecimalNumber *)[NSDecimalNumber numberWithDouble:[aMandelView Magnify]]];
    [aBookmark setShading:(NSDecimalNumber *)[NSDecimalNumber numberWithInt:[[shadingBox selectedCell] tag]]];
    [aBookmark setIterations:(NSDecimalNumber *)[NSDecimalNumber numberWithDouble:[iterationsBox floatValue]]];
    
    [theBookmarks addObject:aBookmark];
    [bookmarksView reloadData];
    [self setBookmarksMenu];
    [self saveBookmarks];
}
- (void)deleteItem:sender {
    [theBookmarks removeObjectAtIndex:(unsigned)[bookmarksView selectedRow]];
    [bookmarksView reloadData];
    [self setBookmarksMenu];
    [self saveBookmarks];
}
- (void)goItem:sender {
    BookmarkItem *aBookmark = [theBookmarks objectAtIndex:(unsigned)[bookmarksView selectedRow]];
    [aMandelView setCwx:[[aBookmark x] doubleValue]];
    [aMandelView setCwy:[[aBookmark y] doubleValue]];
    [aMandelView setMagnify:[[aBookmark magnitude] doubleValue]];
    [iterationsBox setFloatValue:[[aBookmark iterations] floatValue]];
    [shadingBox selectCellWithTag:[[aBookmark valueForKey: @"shading"] intValue]];
    //[MandelView ReCenter:self];
    [[aMandelView window] deminiaturize:self];
    [aMandelView display];
    
}
- (void)saveAsItem:sender {
    [self saveBookmarksAs];
}
- (void)openItem:sender {
    int answer;
    answer=NSRunAlertPanel(NSLocalizedString(@"Open Bookmarks",@"Open Bookmarks,alert title"),
                            NSLocalizedString(@"Do you want to save your bookmarks first?",
                                        @"Save the Current Bookmarks as ( then a new NSSavePanel comes)"),
                            NSLocalizedString(@"Save",@"Save."),NSLocalizedString(@"Cancel",@"Cancel."),
                            NSLocalizedString(@"Don't save",@"Don't save."));
    switch(answer) {
        case 1://User Click Save
            [self saveBookmarksAs];//first save bookmarks
            [self openBookmarks];//then open a new set of
            break;
        case -1://User Click Don't save
            [self openBookmarks];
            break;
       // if User Click Cancel
        //do nothing
    }
}

- (void)toggleMoreInfo:sender {
    if([sender state]==NSOnState) {
        [moreInfo open];
        
        if(![self updateMoreInfoWithMandelView:YES])
                [sender setState:NSOffState];
    }
    else
    {
        [moreInfo close:self];
    }
}
- (BOOL)updateMoreInfoWithMandelView:(BOOL)viewUpdate
{
    if([bookmarksView selectedRow]!=-1) {
            BookmarkItem *aBookmark = [theBookmarks objectAtIndex:(unsigned)[bookmarksView selectedRow]];
            [self updateMoreInfoMandelView];
            [infoNamebox setStringValue: (NSString *)[aBookmark valueForKey: @"name"]];
            [infoybox setFloatValue: (float)[[aBookmark valueForKey: @"y"] floatValue]];
            [infoxbox setFloatValue: (float)[[aBookmark valueForKey: @"x"] floatValue]];
            [infoMagbox setFloatValue: (float)[[aBookmark valueForKey: @"magnitude"] floatValue]];
            [infoIterationsBox setFloatValue: (float)[[aBookmark valueForKey: @"iterations"] floatValue]];
            [infoShadingBox selectCellWithTag:[[aBookmark valueForKey: @"shading"] intValue]];
            if(viewUpdate)
                [self updateMoreInfoMandelView];
            return YES;
        }
        return NO;
    
}
- (void)updateMoreInfoMandelView
{
    BookmarkItem *aBookmark = [theBookmarks objectAtIndex:(unsigned)[bookmarksView selectedRow]];
    [infoMandelView setCwx:[[aBookmark x] doubleValue]];
    [infoMandelView setCwy:[[aBookmark y] doubleValue]];
    [infoMandelView setMagnify:[[aBookmark magnitude] doubleValue]];
    //[MandelView ReCenter:self];
    [infoMandelView display];
}

- (void)updateBookmarkByMoreInfoOptions:sender
{
    BookmarkItem *aBookmark = [theBookmarks objectAtIndex: [bookmarksView selectedRow]];
    [aBookmark setName:(NSString *)[infoNamebox stringValue]];
    [aBookmark setShading:(NSDecimalNumber *)[NSDecimalNumber numberWithInt:[[infoShadingBox selectedCell] tag]]];
    [aBookmark setIterations:(NSDecimalNumber *)[NSDecimalNumber numberWithFloat:[infoIterationsBox floatValue]]];
    
    [bookmarksView reloadData];
    [self setBookmarksMenu];
    [self saveBookmarks];
}
- (void)updateBookmarkByMoreInfoOptionsWithPreviewUpdate:sender
{
    [self updateBookmarkByMoreInfoOptions:sender];
    [self updateMoreInfoMandelView];
}
- (void)goFromBookmarksMenu:(NSMenuItem *)sender
{
    BookmarkItem *aBookmark = [theBookmarks objectAtIndex:[sender tag]];
    [aMandelView setCwx:[[aBookmark x] doubleValue]];
    [aMandelView setCwy:[[aBookmark y] doubleValue]];
    [aMandelView setMagnify:[[aBookmark magnitude] doubleValue]];
    [iterationsBox setFloatValue:[[aBookmark iterations] floatValue]];
    [shadingBox selectCellWithTag:[[aBookmark valueForKey: @"shading"] intValue]];
    //[MandelView ReCenter:self];
    [[aMandelView window] deminiaturize:self];
    [aMandelView display];
}
- (void)setBookmarksMenu
{
        int i;
        for(i=[[Menu submenu] numberOfItems];i>4;i--)
        {
            [[Menu submenu] removeItemAtIndex:i-1];
        }
        for(i=0;i<[[self theBookmarks] count];i++)
        {
            BookmarkItem	*aBookmark=[[self theBookmarks] objectAtIndex:i];
            NSMenuItem		*aMenuItem;
            aMenuItem=[[Menu submenu] addItemWithTitle:[aBookmark valueForKey:@"name"] 
                                                        action:NULL keyEquivalent:@""];
            [aMenuItem setTag:i];
            [aMenuItem setTarget:self];
            [aMenuItem setAction:@selector(goFromBookmarksMenu:)];
            [aMenuItem setEnabled:YES];
        }
}
/*NSTableView delegate stuff*/
- (void)tableViewSelectionDidChange:(NSNotification *)aNotification 
{
    if([moreInfo state]==NSDrawerOpenState) {
        if([self updateMoreInfoWithMandelView:YES]){}
    } else 
        {
            if([bookmarksView selectedRow]==-1)//No row selected
            {
                [moreInfoButton setEnabled:NO];
                [deleteButton setEnabled:NO];
                [goToButton setEnabled:NO];
            }else
                {
                    [moreInfoButton setEnabled:YES];
                    [deleteButton setEnabled:YES];
                    [goToButton setEnabled:YES];
                }
        }
}
@end
